/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.ViewletSite");

dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");

(function(){

var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
	
/*
 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * future with minimal or no warning.
 * 
 * @since 0.6
 */
dojo.declare("com.ibm.team.dashboard.web.ui.ViewletSite",
	null, {
	
	constructor: function(args) {
		this._container = args.container;
		this._viewletHelper = args.viewletHelper;
		this._loading = false;
	},

	/*
	 * Adds a menu item to the viewlet menu.
	 * 
	 * @param label the text shown for the menu item.
	 * @param icon (optional) either an image URL or a css class name specifying
	 * 		a background-image property for the menu item's icon, or null for none.
	 * 		The use of css class is deprecated as it does not work with iframed
	 * 		viewlets (because the css is not available outside the iframe).
	 * @param callback the function to call when the user activates the menu item. The scope
	 * 		(i.e. 'this') will be the viewlet adding the menu item.
	 */
	addMenuItem: function(/* string */ label, /* string */ icon, /* function */ callback) {
		var entry = {label: label, onClick: dojo.hitch(this._container._viewlet, callback)};
		if (icon) {
			// ensure string
			icon = icon + "";
			// check if it's a url or a css class
			if (icon.indexOf("/") != -1) {
				// url iff it has a slash
				entry.icon = icon;
			}
			else {
				// must be a css class; say it's deprecated
				dojo.deprecated("com.ibm.team.dashboard.web.ui.ViewletSite.addMenuItem(label, iconClass, callback)", "Use an icon URL for the second parameter instead of a css class");
				entry.iconClass = icon;
			}
		}
		this._container.addMenuItem(entry);
	},
	
	/*
	 * Returns true or false based on if the current server license level allows viewing of the
	 * indicated dashboard type.
	 * 
	 * @param scope a number representing one of the constants for the scope of the dashboard
	 * @since 0.6.1
	 */
	/* boolean */ canViewDashboard: function(/* number */ scope) {
		var perms = this._container.dashboardBundle.permissions;
		if (scope == DashboardConstants.SCOPE_PROJECT_AREA)
			return perms.viewProject;
		if (scope == DashboardConstants.SCOPE_TEAM_AREA)
			return perms.viewTeam;
		if (scope == DashboardConstants.SCOPE_CONTRIBUTOR) {
			return perms.viewPersonal;
		}
		return false;
	},
	
	/*
	 * Reports a serious error which may impact the normal operation of the
	 * dashboard or other viewlets (e.g. connection failure, server down).
	 * 
	 * @param error An Error object representing the problem.
	 * @since 0.7
	 */
	error: function(/* Error */ error) {
		dojo.publish(Constants.serverError, [error]);
	},
	
	/*
	 * Enables or disables automatic periodic refreshing of viewlets. Viewlets
	 * may wish to temporarily disable automatic refresh if it disrupts the
	 * normal user flow when, for example, the user is reading a hover tooltip or
	 * editing content. Normal auto-refresh will resume as soon as it is re-enabled.
	 */
	setAutoRefresh: function(/* boolean */ value) {
		this._container.setAutoRefresh(value);
	},
	
	/*
	 * Sets the viewlet icon to the image pointed to by the given URL.
	 * 
	 * @param url the URL of the image file.
	 */
	setIcon: function(/* string */ url) {
		this._container.setIcon(url);
	},
	
	/*
	 * Called to set loading state of viewlet.  Displays progress icon when loading.
	 */
	setLoading: function(loadingStatus) {
		if (this._container._iconImage) {
			if (loadingStatus) {
				this._container._iconImage.style.display = "none";
				dojo.addClass(this._container._headerIcon, "com-ibm-team-dashboard-progressIcon");
			}
			else {
				this._container._iconImage.style.display = "inline";
				dojo.removeClass(this._container._headerIcon, "com-ibm-team-dashboard-progressIcon");
			}
		}
		this._loading = loadingStatus;
	},
	
	/*
 	 * Public Method
 	 * 
 	 * Indicates whether the viewlet is loading or not
 	 *  
 	 * @param void
 	 * @return boolean true = loading; false = not loading
 	 */	
	isLoading: function() {
		return this._loading;
	},	
	
	/*
	 * Can be called by viewlets to programatically open or close the settings area.
	 */
	toggleSettings: function() {
		this._container.toggleSettings();
	},
	
	/*
	 * Can be called by viewlets to programatically enter edit mode.
	 */
	editContents: function() {
		this._container.startEditing();
	},
	
	/*
	 * Called by viewlets to update the dynamic title. Some viewlets only know the title
	 * when the SettingsArea is closed. These viewlets should indicate that the title
	 * needs to be persisted by setting the persist flag to true. Note that this results
	 * in a dashboard modification and will make the Dashboard dirty, thus it should
	 * only be done in tandem with another action that dirties the dashboard, such as
	 * changing the viewlet settings.
	 */
	setTitle: function(/* string */ title, /* boolean */ persist) {
		// ignore the call if the user has overriden the title
		if (!this._viewletHelper.userTitle) {
			// update the title on the container
			this._container.setTitle(title, persist);
		}
	},
	
	setSecondaryTitle: function(/* string */ title) {
		this._container.setSecondaryTitle(title);
	},
	
	getTitle: function() {
		return this._viewletHelper.title;
	},
	
	getSecondaryTitle: function() {
		return this._container.getSecondaryTitle();
	},
	
	getActionArea: function() {
		return this._container.actionArea;
	},
	
	isDashboardEditing: function() {
		return this._container.editing;
	},
	
	isActive: function() {
		return this._container.viewletPage.isActive;
	},
	
	/*
 	 * Public Method
 	 * 
 	 * Returns the height of the viewlet's content area
 	 * 
 	 * @param void
 	 * @return Number
 	 */		
	getHeight: function() {
		if (dojo.isIE == 6)
			//directly sniff height for IE6 (dojo.style doesn't support IE6 properly)
			return this._container._contentWrapper.clientHeight;
		else
			return dojo.style(this._container._contentWrapper, "height");
	},
	
	/*
 	 * Public Method
 	 * 
 	 * Returns the width of the viewlet's content area
 	 * 
 	 * @param void
 	 * @return Number
 	 */		
	getWidth: function() {
		if (dojo.isIE == 6) {
			//look at header width instead (because IE6 gives wrong width if content is busting out the side)
			var width = this._container._header.clientWidth - 5; 
			
			//for trimless viewlet case
			if (width < 0) width = this._container._contentWrapper.clientWidth - 40;
			
			return width;
		} else
			return dojo.style(this._container._contentWrapper, "width");
	},
	
	doneRefresh: function() {
		this._container.isRefreshing--;
		if (this._container.isRefreshing < 0)
			this._container.isRefreshing = 0;
	},
	
	refresh: function() {
		this._container.refreshViewlet(true);
	},
	
	navigateAwayEvent: function(eventObj, workbenchNavigation) {
		return this._container.viewletPage.dashboardContainer.dashboardWidget.navigateAwayEvent(eventObj, workbenchNavigation);
	}
});
	
})();
