/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.build.web.ui.internal.query.RowElementViewer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.team.build.web.ui.internal.Constants");
dojo.require("com.ibm.team.build.web.ui.internal.query.CellElementViewer");

(function() {
	
/*
 * Summary: A row widget's parent class representing a "tr" element in tree-table style rendering. This class provides:
 * 			1) handle the click event which transfered from the first cell widget which is the kind that constructs the row widget. The click event
 * 			   will trigger the expand/collapse of the selected row, and so the parent-child relationship can be showed dynamically in tree hierarchy.
 * 			2) interface to get the children row widgets of the selected row widgets.
 * 			3) interface to create the children widget.
 */

var Constants = com.ibm.team.build.web.ui.internal.Constants;
var CellElementViewer = com.ibm.team.build.web.ui.internal.query.CellElementViewer; 

dojo.declare("com.ibm.team.build.web.ui.internal.query.RowElementViewer",
 [dijit._Widget, dijit._Templated],
{     
	// all the rows will be hidden first except those root ones for every tree.
	templateString: "<tr class='hidden'></tr>",
	
	constructor: function(rowArgs) {
		this._parentPage = rowArgs.parentPage;
		this._bodyContainer =rowArgs.container;
		this._parentRowElement = rowArgs.parentRowElement;
		this._resultItemId = rowArgs._resultItemId;
		this._rowData = rowArgs._rowData;
		this._rowType = this._rowData[Constants.REPOSITORY_TYPE];
		this._isLeafRow = false;
	},
	
	postCreate: function() {
		this._childrenRows = [];
		dojo.addClass(this.domNode, "hidden");  //$NON-NLS-1$
		this.initializeViewer();
		this._expanded = false;
	},
	
	// the template method for letting the sub-class to provide more specific rendering for the widget. 	
	initializeViewer: function(){
		return null;
	},
	
	getRowContainer: function() {
		return this.domNode;
	},
	
	addChild: function(childRow){
		var rows = this._childrenRows;
		var prevRow = rows.length == 0 ? this : rows[rows.length-1];
		this._bodyContainer.insertBefore(childRow.domNode, prevRow.domNode.nextSibling);
		rows.push(childRow);
	},
	
	toggleExpand: function(childrenRowsToToggle){
		if(this._expanded){
			this.collapseChildren(childrenRowsToToggle);
			if (this._twistie) {
				dojo.addClass(this._twistie, "twistie-closed");
				dojo.removeClass(this._twistie, "twistie-open");
				dojo.removeClass(this._twistie, "twistie-partial");
			}
		}else{
			this.expandChildren(childrenRowsToToggle);
			if (this._twistie) {
				dojo.addClass(this._twistie, "twistie-open");
				dojo.removeClass(this._twistie, "twistie-closed");
				dojo.removeClass(this._twistie, "twistie-partial");
			}
		}
		this._expanded = ! this._expanded;
	},
	
	showLoading: function() {
		dojo.addClass(this._twistie, "twistie-partial");
		dojo.removeClass(this._twistie, "twistie-open");
		dojo.removeClass(this._twistie, "twistie-closed");
		dojo.addClass(this.domNode.firstChild,"loading-children");
	},
	
	loadFailed: function() {
		dojo.removeClass(this._twistie, "twistie-partial");
		dojo.addClass(this._twistie, "twistie-closed");
		dojo.removeClass(this.domNode.firstChild,"loading-children");
	},
	
	// show the children row widgets for the clicked row, if the children has been expanded/collapsed, that state will be showed as before.
	expandChildren: function(childrenRowsToToggle){
		// only call addChild if the children are new
	    var doAdd = childrenRowsToToggle != this._childrenRows;
		if (childrenRowsToToggle && childrenRowsToToggle.length > 0){
			for (var i= 0; i < childrenRowsToToggle.length; i++) {
				var node= childrenRowsToToggle[i];
				if (doAdd) {
					this.addChild(node);	
				}
				if(dojo.hasClass(node.getRowContainer(),"hidden")){
					dojo.removeClass(node.getRowContainer(), "hidden"); //$NON-NLS-1$
					if(node._expanded){
						node.expandChildren(node.getChildrenRows());
					}
				}					
			}
		}
		if (doAdd) {
			dojo.removeClass(this.domNode.firstChild,"loading-children");
		}
	},
	// hide all the direct/indirect children, and the expanded/collapsed state of those children will not be removed.
	collapseChildren: function(childrenRowsToToggle){
		if (childrenRowsToToggle && childrenRowsToToggle.length > 0){
			for (var i= 0; i < childrenRowsToToggle.length; i++) {
				var node= childrenRowsToToggle[i];
				if(!dojo.hasClass(node,"hidden")){
					dojo.addClass(node.getRowContainer(), "hidden"); //$NON-NLS-1$
					if(node._expanded){
						node.collapseChildren(node.getChildrenRows());
					}
				}
			}
		}
	},
	
	// will be overrided by the sub classes to created the specific rows. 
	createChildRows: function(){  
		return null;
	},
	
	// handle the click event from the first cell of the selected row. If the row has detailed data, they will be displayed in the Details section. Currently,we 
	// just show the details for the leaf node. If not the leaf node, then children rows will be showed or created when this is the first click.
	handleFirstColumnClick: function(){
		if(this.hasDetails()){ //Only show details when the selected row is leaf or has details data itself. 
			this.doShowDetails(this.getDetailType());
			return;
		}
		this.prepareShowDetails();
		var childrenRows = this.getChildrenRows();
		if(childrenRows.length > 0){
			this.toggleExpand(childrenRows);
		}else{
			this.showLoading();
			this.createChildRows();				
		}	
	},
	
	hasDetails: function(){  //Only deal with leaf node's errors, warnings or failures content currently.
		return this.isLeafRow();
	},
	
	doShowDetails: function(detailType){
		return false;
	},
	
	// return the detailed content type, currently supporting Error, Warning and Failure.
	getDetailType: function(){
		return this._detailType;
	},
	
	setDetailType: function(detailType){
		this._detailType = detailType;
	},
	
	prepareShowDetails: function(){
		return false;
	},
	
	getChildrenRows: function(){		
		//Show rows for packages children
		if(this._childrenRows && this._childrenRows.length > 0){
			return this._childrenRows;
		}
		return [];		
	},
	
	// return the data of this row widget.
	getRowData: function(){
		return this._rowData;
	},
	
	// return the type of this row widget, which value will be set in constructing this widget. 
	getRowType: function(){
		return this._rowType;
	},
	
	// return the row's level in the tree.
	getTreeLevel: function(){
		var treeLevel = 0;
		var parentE = this._parentRowElement;
		while(parentE){
			parentE = parentE._parentRowElement
			treeLevel++;
		}
		return treeLevel;
	},
	
	isLeafRow: function(){
		return this._isLeafRow;
	},
	
	destroy: function() {
		this.inherited(arguments);
	}
		
});
	
})();
